--[[
Name: SorrenTimers
Revision: $Rev: 1.5 $
Author(s): Sorren (sorren@astraycat.net)
Website: http://www.wowace.com/
Description: Times skills
Dependencies: AceLibrary
]]
local L = AceLibrary("AceLocale-2.2"):new("SorrenTimers");
local BS = AceLibrary("Babble-Spell-2.2");

local options = {
	type = 'group',
	args = {
		groups = {
			type = 'group',
			name = "Groups Options",
			desc = "Bar Groupings",
			args = {
				list = {
					type = 'execute',
					name = "List",
					desc = "Lists Already Created Groups",
					func = "ListGroups",
				},
				lock = {
					type = 'text',
					name = "Lock",
					desc = "Lock a group Position",
					get = false,
					set = "LockAnchor",
					usage = "<groupname>",
				},
				unlock = {
					type = 'text',
					name = "Unlock",
					desc = "Unlock a group for Positioning",
					get = false,
					set = "UnlockAnchor",
					usage = "<groupname>",
				},
				new = {
					type = 'text',
					name = "new",
					desc = "Make a new group",
					get = false,
					set = "CreateGroup";
					usage = "<groupname>, letters only";
				},
				del = {
					type = 'text',
					name = "del",
					desc = "Remove a group",
					get = false,
					set = "RemoveGroup";
					usage = "<groupname>";
				},
				color = {
					type = 'text',
					name = "Group Colors",
					desc = "Set the default bar color for a group",
					get = false,
					set = "SetGroupColor";
					usage = "<groupname> <color1> <color2> <color3>, <groupname> default to set to default",
				},
				growth = {
					type = 'text',
					name = "Growth Toggle",
					desc = "Toggles whether bars grow upwards or downwards",
					get = false,
					set = "ToggleGrowth",
					usage = "<groupname>",
				},
			},
		},
		skill = {
			type = 'group',
			name = "Skill Options",
			desc = "Skill Options",
			args = {
				list = {
					type = 'text',
					name = "List skills",
					desc = "List skills, shows skill status",
					get = false,
					set = "ListSkills",
					usage = "'all' for all skills, <skill> to get skill status";
				},
				setgroup = {
					type = 'text',
					name = "Set Group",
					desc = "Sets a skill's group",
					get = false,
					set = "SetSkillGroup",
					usage = "<skillname> <groupname>";
				},
				enable = {
					type = 'text',
					name = "Enable ability",
					desc = "Enables tracking for the ability",
					get = false,
					set = "EnableSkill",
					usage = "<skillname>",
				},
				disable = {
					type = 'text',
					name = "Disable ability",
					desc = "Disables tracking for the ability",
					get = false,
					set = "DisableSkill",
					usage = "<skillname>",
				},
				color = {
					type = 'text',
					name = "Skill Colors",
					desc = "Set the default bar color for a skill (overrides group coloring)",
					get = false,
					set = "SetSkillColor";
					usage = "<skillname> <color1> <color2> <color3>, <skillname> default to set to default",
				},
				fade = {
					type = 'text',
					name = "Fade Time",
					desc = "Fade time for skills",
					get = false,
					set = "SetSkillFade",
					usage = "<time in seconds, negative time for sticky, 0 to disable, \"default\" to use the default time>",
				},
				name = {
					type = 'text',
					name = "Display Name",
					desc = "Sets the display name for the skill",
					get = false,
					set = "SetSkillDisplay",
					usage = "<skillname>-<skilldisplayname> or <skillname>-reset to reset";
				},
				targetdisplay = {
					type = 'text',
					name = "Display",
					desc = "Sets how skills with targets will display",
					get = "GetTargetSkillString",
					set = "SetTargetSkillString",
					usage = "<string> use $name for placing the skill name, $target for placing skill target, \"empty\" to show an empty bar, ex $name - $target",
				},
				display = {
					type = 'text',
					name = "Display",
					desc = "Sets how skills without targets will display",
					get = "GetSkillString",
					set = "SetSkillString",
					usage = "<string> use $name for placing the skill name, \"empty\" to show an empty bar",
				},
				diminishdisplay = {
					type = 'text',
					name = "Diminishing Display",
					desc = "Sets how diminishing returns will display",
					get = "GetDiminishString",
					set = "SetDiminishString",
					usage = "<string> use $name for placing the diminishing type, $target for the target, and $casts for how many times you can cast again. \"empty\" to show an empty bar",
				},
				diminishcolor = {
					type = 'text',
					name = "Diminishing Colors",
					desc = "Default colors for diminishing return bars",
					get = "GetDiminishColors",
					set = "SetDiminishColors",
					usage = "<color1>, <color2>, <color3>"
				},
				diminishgroup = {
					type = 'text',
					name = "Diminishing Group",
					desc = "Default skill group for diminishing return bars",
					get = "GetDiminishGroup",
					set = "SetDiminishGroup",
					usage = "<groupname>, /st skill diminishgroup reset to set it to use the group of the skill which triggered it",
				},
			},
		},
		enemyskill = {
			type = 'group',
			name = "Skill Options",
			desc = "Skill Options",
			args = {
				list = {
					type = 'text',
					name = "List skills",
					desc = "List skills, shows skill status",
					get = false,
					set = "ListEnemySkills",
					usage = "'all' for all skills, <skill> to get skill status";
				},
				setgroup = {
					type = 'text',
					name = "Set Group",
					desc = "Sets a skill's group",
					get = false,
					set = "SetEnemySkillGroup",
					usage = "<skillname> <groupname>";
				},
				enable = {
					type = 'text',
					name = "Enable ability",
					desc = "Enables tracking for the ability",
					get = false,
					set = "EnableEnemySkill",
					usage = "<skillname>",
				},
				disable = {
					type = 'text',
					name = "Disable ability",
					desc = "Disables tracking for the ability",
					get = false,
					set = "DisableEnemySkill",
					usage = "<skillname>",
				},
				color = {
					type = 'text',
					name = "Skill Colors",
					desc = "Set the default bar color for a skill (overrides group coloring)",
					get = false,
					set = "SetEnemySkillColor";
					usage = "<skillname> <color1> <color2> <color3>, <skillname> default to set to default",
				},
				fade = {
					type = 'text',
					name = "Fade Time",
					desc = "Fade time for skills",
					get = false,
					set = "SetEnemySkillFade",
					usage = "<time in seconds, negative time for sticky, 0 to disable, \"default\" to use the default time>",
				},
				name = {
					type = 'text',
					name = "Display Name",
					desc = "Sets the display name for the skill",
					get = false,
					set = "SetEnemySkillDisplay",
					usage = "<skillname>-<skilldisplayname> or <skillname>-reset to reset";
				},
				display = {
					type = 'text',
					name = "Display",
					desc = "Sets how skills will display",
					get = "GetEnemySkillString",
					set = "SetEnemySkillString",
					usage = "<string> use $name for placing the skill name, $target for the enemy name, \"empty\" to show an empty bar, ex: $name - $target",
				},
			},
		},
		bar = {
			type = 'group',
			name = "bar",
			desc = "bar settings",
			args = {
				fade = {
					type = 'text',
					name = "Fade Time",
					desc = "Fade time for bars",
					get = "GetBarFade",
					set = "SetBarFade",
					usage = "<time in seconds, negative time for sticky, 0 to disable>",
				},
				scale = {
					type = 'text',
					name = "Bar Scale",
					desc = "Set Bar Scales",
					get = "GetBarScale",
					set = "SetBarScale",
					usage = "<scale 0.5 to 2, may require a reloadui>",
				},
				color = {
					type = 'text',
					name = "Default bar colors",
					desc = "Set the default bar colors",
					get = "GetBarColors",
					set = "SetBarColors",
					usage = "<color1> <color2> <color3> (may require reloadui)",
				},
			},
		},
		bgtimers = {
			type = 'toggle',
			name = "Battleground Timers",
			desc = "Toggle Battlegrounds (AB & Arena) Timers (not fully implemented)",
			get = "GetBGEnable",
			set = "SetBGEnable",
			map = { [false] = "Disabled", [true] = "Enabled" },
		},
		showicon = {
			type = 'toggle',
			name = "Show Minimap Icon",
			desc = "Toggle the minimap icon",
			get = "GetIcon",
			set = "SetIcon",
			map = { [false] = "Hidden", [true] = "Shown" },
		},
	},
};
local barQueue, barTarget, afflictSpell, numGroups, pc, comboPoints;
local units = { };
local gcdstart = 0;
skillSet = {};
local auras = { };
local nodes = {
	["lumber mill"] = { 
		["name"] = "Lumber Mill",
		["num"] = 4,
	},
	["stables"] = {
		["name"] = "Stables",
		["num"] = 5,
	},
	["farm"] = {
		["name"] = "Farm",
		["num"] = 2,
	},
	["blacksmith"] = {
		["name"] = "Blacksmith",
		["num"] = 1,
	},
	["mine"] = {
		["name"] = "Mine",
		["num"] = 3,
	},
};
local interruptSkill;
local barRank;
local gcdspell;
SorrenTimers = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0", "CandyBar-2.0", "AceHook-2.1", "AceDebug-2.0");
SorrenTimers:RegisterChatCommand({"/sorrentimers", "/st"}, options);
SorrenTimers:RegisterDB("SorrenTimersDB", "SorrenTimersDBPC");
SorrenTimers:RegisterDefaults("char", {
	["groups"] = {
		["defaultgroup"] = { },
	},
	["defaultcolors"] = {"blue", "blue", "blue"};
	["diminish"] = {
		["colors"] = {"red", "red", "red"},
		["skillstring"] = "$casts - $name - $target",
	},
	["barscale"] = 1;
	["barfade"] = 0;
	["skilltargetstring"] = "$name - $target",
	["skillstring"] = "$name",
	["enemyskillstring"] = "$name - $target",
	["debugging"] = false,
	["bgSettings"] = {
		["enabled"] = false,
		["horde"] = { "red", "red", "red" },
		["alliance"] = { "blue", "blue", "blue" },
		["neutral"] = { "green", "green", "green" },
	},
	["showicon"] = true,
} );

function SorrenTimers:GetIcon()
	if not self.icon then return false; end
	return self.db.char.showicon;
end

function SorrenTimers:SetIcon()
	if not self.icon then return; end
	self.db.char.showicon = not self.db.char.showicon;
	if self.db.char.showicon then
		self.icon:Show();
	else
		self.icon:Hide();
	end
end
	
function SorrenTimers:GetBGEnable()
	return self.db.char.bgSettings.enabled;
end

function SorrenTimers:SetBGEnable()
	self.db.char.bgSettings.enabled = not self.db.char.bgSettings.enabled;
	if self.db.char.bgSettings.enabled then
		self:RegisterEvent("CHAT_MSG_BG_SYSTEM_NEUTRAL");
		self:RegisterEvent("CHAT_MSG_BG_SYSTEM_HORDE");
		self:RegisterEvent("CHAT_MSG_BG_SYSTEM_ALLIANCE");
	else
		self:UnregisterEvent("CHAT_MSG_BG_SYSTEM_NEUTRAL");
		self:UnregisterEvent("CHAT_MSG_BG_SYSTEM_HORDE");
		self:UnregisterEvent("CHAT_MSG_BG_SYSTEM_ALLIANCE");
	end
end

function SorrenTimers:ResetDiminishGroup(diminish)
	self.db.char.abilities[diminish].group = nil;
	self:Print(diminish.." set to use default Diminishing group");
end

function SorrenTimers:GetDiminishGroup()
	if not self.db.char.diminish.group then return "Group of the Skill which Triggers it" end;
	return self.db.char.diminish.group;
end

function SorrenTimers:GetDiminishColors()
	return self.db.char.diminish.colors[1].." "..
		self.db.char.diminish.colors[2].." "..
		self.db.char.diminish.colors[3];
end

function SorrenTimers:GetDiminishString()
	return self.db.char.diminish.skillstring;
end

function SorrenTimers:SetDiminishGroup(group)
	if group ~= "-reset" and not self.db.char.groups[group] then
		self:Print("Group "..group.." not found!");
		return;
	end
	if group == "-reset" then self.db.char.diminish.group = nil; return; end
	self.db.char.diminish.group = group;
end

function SorrenTimers:SetDiminishColors(colorstring)
	for color1, color2, color3 in string.gmatch(colorstring, "(%a+) (%a+) (%a+)$") do
		self.db.char.diminish.colors = { color1, color2, color3 };
		return;
	end
	self:Print("Example: /st skill diminishcolor red red red");
end

function SorrenTimers:SetDiminishString(skillstring)
	if skillstring ~= "empty" then
		self.db.char.diminish.skillstring = skillstring;
	else
		self.db.char.diminish.skillstring = "";
	end
end

function SorrenTimers:GetBarColors()
	local colors = "";
	for k,v in pairs(self.db.char.defaultcolors) do
		colors = colors..v.." ";
	end
	return colors;
end

function SorrenTimers:SetBarColors(colors)
	for color1, color2, color3 in string.gmatch(colors, "(%a+) (%a+) (%a+)") do
		self.db.char.defaultcolors = {color1, color2, color3};
		self:Print("Default Colors set to: "..color1.." "..color2.." "..color3);
		return;
	end
	self:Print("Example: /st bar color red green blue");
end

function SorrenTimers:GetBarScale()
	return self.db.char.barscale;
end

function SorrenTimers:SetBarScale(scale)
	scale = tonumber(scale);
	if scale < 0.5 or scale > 2 then
		self:print("Valid scales are between 0.5 (50%) and 2 (200%).");
		return;
	end
	self.db.char.barscale = scale;
	self:Print("Bar scale set to: "..scale);
end

function SorrenTimers:GetBarFade()
	return self.db.char.barfade;
end

function SorrenTimers:SetBarFade(seconds)
	seconds = tonumber(seconds);
	self.db.char.barfade = seconds;
	self:Print("Fade time set to "..seconds.." seconds.");
end

function SorrenTimers:SetSkillFade(fadestring)
	for skill, seconds in string.gmatch(fadestring, "(.+) (-?%d+%.?%d*)$") do
		seconds = tonumber(seconds);
		if not self.db.char.abilities[skill] then
			self:Print("Skill: "..skill.." not found!");
			return
		end
		self.db.char.abilities[skill].fade = seconds;
		self:Print("Fade time for "..skill.." set to "..seconds.." seconds.");
		return;
	end
	for skill in string.gmatch(fadestring, "(.+) default$") do
		if not self.db.char.abilities[skill] then
			self:Print("Skill: "..skill.." not found!");
			return;
		end
		self.db.char.abilities[skill].fade = nil;
		self:Print("Fade time for "..skill.." set to default.");
		return;
	end
	self:Print("Invalid string. Usage: /st skill fade <skillname> <fade time in seconds> or /st skill fade <skillname> default");
end

function SorrenTimers:LockAnchor(group)
	if not self.db.char.groups[group] then self:Print("Group "..group.." not found!"); return end
	self.anchors[group]:Hide();
	self.db.char.anchors["STAnchor"..group].unlocked = false;
end

function SorrenTimers:UnlockAnchor(group)
	if not self.db.char.groups[group] then self:Print("Group "..group.." not found!"); return end
	self.anchors[group]:Show();
	self.db.char.anchors["STAnchor"..group].unlocked = true;
end

function SorrenTimers:ListGroups()
	local groups;
	for group,k in pairs(self.db.char.groups) do
		if not groups then
			groups = "Group Listing: { "..group.." ";
		else
			groups = groups.."| "..group.." ";
		end
	end
	groups = groups.."}";
	self:Print(groups);
end

function SorrenTimers:ToggleGrowth(group)
	if not self.db.char.groups[group] then self:Print("Group "..group.." not found!"); return end
	self.db.char.groups[group].up = not self.db.char.groups[group].up;
	self:SetCandyBarGroupGrowth(group, self.db.char.groups[group].up);
	if self.db.char.groups[group].up then
		self:Print("Growth set to: Up");
	else
		self:Print("Growth set to: Down");
	end
end

function SorrenTimers:ListSkills(skill)
	if skill ~= "all" then
		if self.db.char.abilities[skill] then
			self:Print(skill);
			if self.db.char.abilities[skill].enable ~= nil then
				local enable = "false";
				if self.db.char.abilities[skill].enable then enable = "true"; end
				self:Print("\tEnabled: "..enable);
			end
			if( self.db.char.abilities[skill].group ) then
				self:Print("\tGroup: "..self.db.char.abilities[skill].group );
			end
		else
			self:Print(skill.." not found.");
		end
		return;
	end
	local skills;
	for skill in pairs(self.db.char.abilities) do
		if not skills then
			skills = "Skills: { "..skill.." ";
		else
			skills = skills.."| "..skill.." ";
		end
	end
	skills = skills.."}";
	self:Print(skills);
end

function SorrenTimers:SetSkillDisplay(skillstring)
	for skill, display in string.gmatch(skillstring, "([^-]+)-(.+)$") do
		if not self.db.char.abilities[skill] then
			self:Print("Skill: "..skill.." not found!");
			return;
		end
		if display == "reset" then
			self:Print(skill.." display reset.");
			self.db.char.abilities[skill].name = nil;
			return;
		end
		self.db.char.abilities[skill].name = display;
		self:Print(skill.." display set to: "..display);
	end
end

function SorrenTimers:SetEnemySkillDisplay(skillstring)
	for skill, display in string.gmatch(skillstring, "([^-]+)-(.+)$") do
		if not self.db.char.enemy[skill] then
			self:Print("Skill: "..skill.." not found!");
			return;
		end
		if display == "reset" then
			self:Print(skill.." display reset.");
			self.db.char.enemy[skill].name = nil;
			return;
		end
		self.db.char.enemy[skill].name = display;
		self:Print(skill.." display set to: "..display);
	end
end

function SorrenTimers:SetEnemySkillFade(fadestring)
	for skill, seconds in string.gmatch(fadestring, "(.+) (-?%d+%.?%d*)$") do
		seconds = tonumber(seconds);
		if not self.db.char.enemy[skill] then
			self:Print("Skill: "..skill.." not found!");
			return
		end
		self.db.char.enemy[skill].fade = seconds;
		self:Print("Fade time for "..skill.." set to "..seconds.." seconds.");
		return;
	end
	for skill in string.gmatch(fadestring, "(.+) default$") do
		if not self.db.char.enemy[skill] then
			self:Print("Skill: "..skill.." not found!");
			return;
		end
		self.db.char.enemy[skill].fade = nil;
		self:Print("Fade time for "..skill.." set to default.");
		return;
	end
	self:Print("Invalid string. Usage: /st skill fade <skillname> <fade time in seconds> or /st skill fade <skillname> default");
end

function SorrenTimers:GetTargetSkillString()
	return self.db.char.skilltargetstring;
end

function SorrenTimers:SetTargetSkillString(skillstring)
	if skillstring == "empty" then skillstring = ""; end
	self.db.char.skilltargetstring = skillstring;
end

function SorrenTimers:GetSkillString()
	return self.db.char.skillstring;
end

function SorrenTimers:SetSkillString(skillstring)
	if skillstring == "empty" then skillstring = ""; end
	self.db.char.skillstring = skillstring;
end

function SorrenTimers:GetEnemySkillString()
	return self.db.char.enemyskillstring;
end

function SorrenTimers:SetEnemySkillString(skillstring)
	if skillstring == "empty" then skillstring = ""; end
	self.db.char.enemyskillstring = skillstring;
end

function SorrenTimers:ListEnemySkills(skill)
	if skill ~= "all" then
		if self.db.char.enemy[skill] then
			self:Print(skill);
			if self.db.char.enemy[skill].enable ~= nil then
				local enable = "false";
				if self.db.char.enemy[skill].enable then enable = "true"; end
				self:Print("\tEnabled: "..enable);
			end
			if( self.db.char.enemy[skill].group ) then
				self:Print("\tGroup: "..self.db.char.enemy[skill].group );
			end
		else
			self:Print(skill.." not found.");
		end
		return;
	end
	local skills;
	for skill in pairs(self.db.char.enemy) do
		if not skills then
			skills = "Skills: { "..skill.." ";
		else
			skills = skills.."| "..skill.." ";
		end
	end
	skills = skills.."}";
	self:Print(skills);
end

function SorrenTimers:SetEnemySkillGroup(skillstring)
	for skill, group in string.gmatch(skillstring, "(.+) (%a+)$") do
		if not self.db.char.enemy[skill] then
			self:Print("Skill "..skill.." not found!");
			return;
		end
		if not self.db.char.groups[group] then
			self:Print("Group "..group.." not found!");
			return;
		end
		if not self.db.char.enemy[skill].group then
			self:Print("This ability ("..skill..") does not have a bar. Try looking for the effect name instead.");
		end
		self.db.char.enemy[skill].group = group;
		self:Print(skill.." set to group "..group);
		if self:IsCandyBarRegistered(skill) then
			self:RegisterCandyBarWithGroup(skill, group);
		end
		return;
	end
end

function SorrenTimers:EnableEnemySkill(skill)
	if not self.db.char.enemy[skill] then
		self:Print("Skill "..skill.." not found!");
		return;
	end
	if self.db.char.enemy[skill].enable == nil then
		self:Print(skill.." cannot be enable or diabled. Try the ability instead.");
		return;
	end
	if self.db.char.enemy[skill].enable then
		self:Print(skill.." already enabled.");
	end
	self.db.char.enemy[skill].enable = true;
	self:Print(skill.." enabled.");
end

function SorrenTimers:DisableEnemySkill(skill)
	if not self.db.char.enemy[skill] then
		self:Print("Skill "..skill.." not found!");
		return;
	end
	if self.db.char.enemy[skill].enable == nil then
		self:Print(skill.." cannot be disabled or disabled. Try the ability instead.");
		return;
	end
	if not self.db.char.enemy[skill] then
		self:Print(skill.." already disabled.");
	end
	self:Print(skill.." disabled.");
	self.db.char.enemy[skill].enable = false;
end

function SorrenTimers:CreateGroup(group)
	for _ in string.gmatch(group, "%A") do
		self:Print("Group names must consist of letters only.");
		return;
	end
	if self.db.char.groups[group] then
		self:Print("There is already a group with the name: "..group);
		return;
	end
	self.db.char.groups[group] = {
		["up"] = false;
	};
	self:CreateAnchor(group);
	self:RegisterCandyBarGroup(group);
	self:SetCandyBarGroupPoint(group, "TOP", self.anchors[group]:GetName(), "BOTTOM", 0, 0);
end

function SorrenTimers:RemoveGroup(group)
	if not self.db.char.groups[group] then
		self:Print("Group "..group.." not found!");
		return;
	end
	if group == "defaultgroup" then
		self:Print("You cannot remove the default group.");
		return;
	end
	self.db.char.groups[group] = nil;
	self.anchors[group]:Hide();
	self.db.char.anchors["STAnchor"..group] = nil;
end

function SorrenTimers:SetSkillGroup(skillstring)
	for skill, group in string.gmatch(skillstring, "(.+) (%a+)$") do
		if not self.db.char.abilities[skill] then
			self:Print("Skill "..skill.." not found!");
			return;
		end
		if not self.db.char.groups[group] then
			self:Print("Group "..group.." not found!");
			return;
		end
		if not self.db.char.abilities[skill].group then
			self:Print("This ability ("..skill..") does not have a bar. Try looking for the effect name instead.");
		end
		self.db.char.abilities[skill].group = group;
		self:Print(skill.." set to group "..group);
		if self:IsCandyBarRegistered(skill) then
			self:RegisterCandyBarWithGroup(skill, group);
		end
		return;
	end
end

function SorrenTimers:EnableSkill(skill)
	if not self.db.char.abilities[skill] then
		self:Print("Skill "..skill.." not found!");
		return;
	end
	if self.db.char.abilities[skill].enable == nil then
		self:Print(skill.." cannot be enable or diabled. Try the ability instead.");
		return;
	end
	if self.db.char.abilities[skill].enable then
		self:Print(skill.." already enabled.");
	end
	self.db.char.abilities[skill].enable = true;
	self:Print(skill.." enabled.");
end

function SorrenTimers:DisableSkill(skill)
	if not self.db.char.abilities[skill] then
		self:Print("Skill "..skill.." not found!");
		return;
	end
	if self.db.char.abilities[skill].enable == nil then
		self:Print(skill.." cannot be disabled or disabled. Try the ability instead.");
		return;
	end
	if not self.db.char.abilities[skill] then
		self:Print(skill.." already disabled.");
	end
	self:Print(skill.." disabled.");
	self.db.char.abilities[skill].enable = false;
end

function SorrenTimers:UNIT_SPELLCAST_SUCCEEDED()
	if( arg1 ~= "player" ) then return; end
	if( skillSet.useActionSkills[arg2] and self.db.char.abilities[arg2].enable ) then
		if skillSet.useActionSkills[arg2].interrupt then
			interruptSkill = skillSet.useActionSkills[arg2];
			return;
		end		
		if( skillSet.useActionSkills[arg2].duration or skillSet.useActionSkills[arg2].refresh or skillSet.useActionSkills[arg2].stop or skillSet.useActionSkills[arg2].ranks or skillSet.useActionSkills[arg2].points ) then
			self:SetUpBar(skillSet.useActionSkills[arg2], arg2, barTarget, UnitIsPlayer("target"), arg3);
		end 
		if( skillSet.useActionSkills[arg2].afflict ) then
			if type(skillSet.useActionSkills[arg2].afflict) == 'table' then
				for k,v in pairs(skillSet.useActionSkills[arg2].afflict) do
					skillSet.debuffs[v].enable = GetTime();
					if skillSet.debuffs[v].delay then
						skillSet.debuffs[v].enable = skillSet.debuffs[v].enable + skillSet.debuffs[v].delay;
					end
				end
			else
				skillSet.debuffs[skillSet.useActionSkills[arg2].afflict].enable = GetTime();
				if skillSet.debuffs[skillSet.useActionSkills[arg2].afflict].delay then
					skillSet.debuffs[skillSet.useActionSkills[arg2].afflict].enable = skillSet.debuffs[skillSet.useActionSkills[arg2].afflict].enable + skillSet.debuffs[skillSet.useActionSkills[arg2].afflict].delay;
				end
			end
			barRank = arg3;
		end
		if( skillSet.useActionSkills[arg2].aoeafflict ) then
			if type(skillSet.useActionSkills[arg2].aoeafflict) == 'table' then
				for k,v in pairs(skillSet.useActionSkills[arg2].aoeafflict) do
					skillSet.aoeDebuffs[v].enable = GetTime();
					if skillSet.aoeDebuffs[v].delay then
						skillSet.aoeDebuffs[v].enable = skillSet.aoeDebuffs[v].enable + skillSet.aoeDebuffs[v].delay;
					end
				end
			else
				skillSet.aoeDebuffs[skillSet.useActionSkills[arg2].aoeafflict].enable = GetTime();
				if skillSet.aoeDebuffs[skillSet.useActionSkills[arg2].aoeafflict].delay then
					skillSet.aoeDebuffs[skillSet.useActionSkills[arg2].aoeafflict].enable = skillSet.aoeDebuffs[skillSet.useActionSkills[arg2].aoeafflict].enable + skillSet.aoeDebuffs[skillSet.useActionSkills[arg2].aoeafflict].delay;
				end
			end
			barRank = arg3;
			aoeCharges = 0;
		end
		if( skillSet.useActionSkills[arg2].buff ) and UnitExists("target") and not UnitIsUnit("target", "player") then
			skillSet.buffs[skillSet.useActionSkills[arg2].buff].enable = GetTime();
			skillSet.buffs[skillSet.useActionSkills[arg2].buff].target = UnitName("target");
			barRank = arg3;
		end
	end
	if self.db.char.abilities[BS["Auto Shot"]] and (arg2 == BS["Auto Shot"] or arg2 == BS["Aimed Shot"] ) then
		if not self.db.char.abilities[BS["Auto Shot"]] or not self.db.char.abilities[BS["Auto Shot"]].active then
			return;
		end

		self.db.char.abilities[BS["Auto Shot"]].duration = UnitRangedDamage("player");
		if( self:IsCandyBarRegistered(BS["Auto Shot"]) )then
			self:SetCandyBarTime(BS["Auto Shot"], self.db.char.abilities[BS["Auto Shot"]].duration);
		end
		self.db.char.abilities[BS["Auto Shot"]].icon = GetInventoryItemTexture("player", 18);
		self:SetUpBar(self.db.char.abilities[BS["Auto Shot"]], BS["Auto Shot"]);
		return;
	end
end

function Clean(action, target, diminishing)
	if units[target] then
		if diminishing and units[target][diminishing] then
			if SorrenTimers:IsCandyBarRegistered(diminishing.."-diminish-"..target) then
				SorrenTimers:StartCandyBar(diminishing.."-diminish-"..target);
			end
		end
		if units[target][action] then
			units[target][action] = nil;
		end
		if not next(units[target]) then
			units[target] = nil;
		end
	end	
end


function SorrenTimers:SetGroupColor(colors)
	for group, default in string.gmatch(colors, "(%a+) default$") do
		if not self.db.char.groups[group] then
			self:Print("Group: "..group.." not found.");
			return;
		end
		self.db.char.groups[group].color = nil;
		self:Print("Colors for "..group.." set to default colors.");
		return;
	end
		
	for group, color1, color2, color3 in string.gmatch(colors, "(%a+) (%a+) (%a+) (%a+)") do
		if not self.db.char.groups[group] then
			self:Print("Group: "..group.." not found.");
			return;
		end
		self.db.char.groups[group].color = { color1, color2, color3 };
		self:Print("Colors for "..group.." set to: "..color1..", "..color2..", "..color3);
		return;
	end
	self:Print("Example: /st groups color defaultgroup red green blue");
end

function SorrenTimers:SetSkillColor(colors)
	for skillName, default in string.gmatch( colors, "(.+) default$") do
		if not self.db.char.abilities[skillName] then
			self:Print("Skill: "..skillName.." not found.");
			return;
		end
		self.db.char.abilities[skillName].color = nil;
		self:Print("Colors for "..skillName.." set to default colors.");
		if self:IsCandyBarRegistered(skillName) then
			self:UnregisterCandyBar(skillName);
		end
		return;
	end
	for skillName, color1, color2, color3 in string.gmatch(colors, "(.+) (%a+) (%a+) (%a+)") do
		if not self.db.char.abilities[skillName] then
			self:Print("Skill: "..skillName.." not found.");
			return;
		end
		self.db.char.abilities[skillName].color = { color1, color2, color3 };
		self:Print("Colors for "..skillName.." set to: "..color1..", "..color2..", "..color3);
		if self:IsCandyBarRegistered(skillName) then
			self:UnregisterCandyBar(skillName);
		end
		return;
	end
	self:Print("Example: /st skill color Freezing Trap Effect blue green yellow");
end

function SorrenTimers:SetEnemySkillColor(colors)
	for skillName, default in string.gmatch( colors, "(.+) default$") do
		if not self.db.char.enemy[skillName] then
			self:Print("Skill: "..skillName.." not found.");
			return;
		end
		self.db.char.enemy[skillName].color = nil;
		self:Print("Colors for "..skillName.." set to default colors.");
		if self:IsCandyBarRegistered(skillName) then
			self:UnregisterCandyBar(skillName);
		end
		return;
	end
	for skillName, color1, color2, color3 in string.gmatch(colors, "(.+) (%a+) (%a+) (%a+)") do
		if not self.db.char.enemy[skillName] then
			self:Print("Skill: "..skillName.." not found.");
			return;
		end
		self.db.char.enemy[skillName].color = { color1, color2, color3 };
		self:Print("Colors for "..skillName.." set to: "..color1..", "..color2..", "..color3);
		if self:IsCandyBarRegistered(skillName) then
			self:UnregisterCandyBar(skillName);
		end
		return;
	end
	self:Print("Example: /st skill color Freezing Trap Effect blue green yellow");
end

function SorrenTimers:OnInitialize()
end

function SorrenTimers:OnEnable()
	self:RegisterEvent("SKILL_LINES_CHANGED");
end

function SorrenTimers:SKILL_LINES_CHANGED()
	self:SetDebugging(self.db.char.debugging);
	local _, class = UnitClass("player");
	self:Print(class);
	if( class == "HUNTER" ) then
		skillSet, gcdspell = stSetHunter();
	elseif( class == "SHAMAN" ) then
		skillSet, gcdspell = stSetShaman();
	elseif( class == "DRUID" ) then
		skillSet, gcdspell = stSetDruid();
		self:RegisterEvent("PLAYER_COMBO_POINTS");
		self.comboPoints = 0;
	elseif( class == "WARRIOR" ) then
		skillSet, gcdspell = stSetWarrior();
	elseif( class == "ROGUE" ) then
		skillSet, gcdspell = stSetRogue();
		self:RegisterEvent("PLAYER_COMBO_POINTS");
		self.comboPoints = 0;
	elseif( class == "PRIEST" ) then
		skillSet, gcdspell = stSetPriest();
	elseif( class == "WARLOCK" ) then
		skillSet, gcdspell = stSetWarlock();
	elseif( class == "MAGE" ) then
		skillSet, gcdspell = stSetMage();
	elseif( class == "PALADIN" ) then
		skillSet, gcdspell = stSetPaladin();
	end
	
	if not self.db.char.abilities then
		self.db.char.abilities = { };
	end
	self:UnregisterEvent("SKILL_LINES_CHANGED");

	self:GetGlobalCooldownSpell();
	for setName, set in pairs(skillSet) do
		for skillName, skill in pairs(set) do
			if not self.db.char.abilities[skillName] then
				self.db.char.abilities[skillName] = { };
				self.db.char.abilities[skillName].enable = true;
				if setName ~= "diminish" then
					self.db.char.abilities[skillName].group = "defaultgroup";
				end
			end
			if setName == "diminish" then
				self.db.char.abilities[skillName].diminish = true;
			end
			if skill.group then
				skill.group = self.db.char.abilities[skillName].group;
			end
			if skill.enable then
				skill.enable = self.db.char.abilities[skillName].enable;
			end
			if skill.level then
				if UnitLevel("player") >= skill.level then
					skill.enable = self.db.char.abilities[skillName].enable;
				end
			end
			if skill.name then
				if not self.db.char.abilities[skillName].name then
					self.db.char.abilities[skillName].name = skill.name;
				end
			end
			if skill.talent then
				if not skill.talent.multiple then
					local _, _, _, _, points = GetTalentInfo(skill.talent.tab, skill.talent.num);
					self:Debug(GetTalentInfo(skill.talent.tab, skill.talent.num));
					if skill.talent.duration then
						if skill.duration then
							skill.duration = (points*skill.talent.duration) + skill.duration;
							self:Debug(skillName,"Talent Points: "..points);
						end
						if skill.ranks then
							for k,v in pairs(skill.ranks) do
								v = (points*skill.talent.duration) + v;
							end
						end
					end
					if skill.talent.durmult then
						if skill.duration then
							skill.duration = (1 + points*skill.talent.durmult) * skill.duration;
						end
						if skill.ranks then
							for k,v in pairs(skill.ranks) do
								v = (1 + points*skill.talent.durmult) * v;
							end
						end
					end
					if skill.talent.enable then 
						if points ~= 0 then
							skill.enable = self.db.char.abilities[skillName].enable;
						else
							skill.enable = false;
						end
					end
					if skill.talent.afflict and points ~= 0 then
						if not skill.afflict then skill.afflict = {}; end
						table.insert(skill.afflict, skill.talent.afflict);
					end
				else
					for k,talent in pairs(skill.talent.multiple) do 
						local _, _, _, _, points = GetTalentInfo(talent.tab, talent.num);
						if talent.duration then
							if skill.duration then
								skill.duration = (points*talent.duration) + skill.duration;
							end
							if skill.ranks then
								for _,v in pairs(skill.ranks) do
									v = v + (points*talent.duration);
								end
							end
						end
						if talent.durmult then
							if skill.duration then
								skill.duration = (1 + points*talent.durmult) * skill.duration;
							end
							if skill.ranks then
								for _,v in pairs(skill.ranks) do
									v = v * (1 + points*talent.durmult);
								end
							end
						end
						if talent.enable then 
							if points ~= 0 then
								skill.enable = self.db.char.abilities[skillName].enable;
							else
								skill.enable = false;
							end
						end
						if talent.afflict and points ~= 0 then
							if not skill.afflict then skill.afflict = {}; end
							table.insert(skill.afflict, talent.afflict);
						end
					end
				end
			end
		end
	end
	
	if not self.db.char.enemy then self.db.char.enemy = {}; end
	
	for buffName, buff in pairs(enemySkills) do
		if not self.db.char.enemy[buffName] then
			self.db.char.enemy[buffName] = {};
			self.db.char.enemy[buffName].enable = true;
			self.db.char.enemy[buffName].group = "defaultgroup";
		end
		buff.enable = self.db.char.enemy[buffName].enable;
		buff.group = self.db.char.enemy[buffName].group;
	end
	--make everything work >.>
	if not skillSet.onDodge then skillSet.onDodge = {}; end
	if not skillSet.onParry then skillSet.onParry = {}; end
	if not skillSet.onBlock then skillSet.onBlock = {}; end
	if not skillSet.onEnemyDodge then skillSet.onEnemyDodge = {}; end
	if not skillSet.onEnemyParry then skillSet.onEnemyParry = {}; end
	if not skillSet.aoeDebuffs then skillSet.aoeDebuffs = {}; end
	if not skillSet.canDie then skillSet.canDie = {}; end
	if not skillSet.buffs then skillSet.buffs = {}; end
	if not skillSet.onKill then skillSet.onKill = {}; end
	if not skillSet.onCrit then skillSet.onCrit = {}; end
	if not skillSet.onDebuff then skillSet.onDebuff = {}; end
	if not skillSet.onSelfBuff then skillSet.onSelfBuff = {}; end
	
	if self.db.char.bgSettings.enabled then
		if not self.db.char.groups["BGTimers"] then self.db.char.groups["BGTimers"] = {}; end
		self:RegisterEvent("CHAT_MSG_BG_SYSTEM_NEUTRAL");
		self:RegisterEvent("CHAT_MSG_BG_SYSTEM_HORDE");
		self:RegisterEvent("CHAT_MSG_BG_SYSTEM_ALLIANCE");
	end
	for group,k in pairs(self.db.char.groups) do
		if type(k) == 'boolean' then
			k = {
				["up"] = false,
			};
		end
		if k.up == nil then
			k.up = false;
		end
		self:CreateAnchor(group);
		self:RegisterCandyBarGroup(group);
		self:SetCandyBarGroupPoint(group, "TOP", self.anchors[group]:GetName(), "BOTTOM", 0, 0)
		self:SetCandyBarGroupGrowth(group, k.up);
	end
	
	if class == "HUNTER" then
		if not self.db.char.abilities[BS["Auto Shot"]] then
			self.db.char.abilities[BS["Auto Shot"]] = {};
			self.db.char.abilities[BS["Auto Shot"]].enable = true;
			self.db.char.abilities[BS["Auto Shot"]].group = "defaultgroup";
		end
		self.db.char.abilities[BS["Auto Shot"]].multiple = nil;
		self:RegisterEvent("START_AUTOREPEAT_SPELL");
		self:RegisterEvent("STOP_AUTOREPEAT_SPELL");
	end
	if not self.db.char.abilities[L["Global Cooldown"]] then
		self.db.char.abilities[L["Global Cooldown"]] = {};
		self.db.char.abilities[L["Global Cooldown"]].enable = true;
		self.db.char.abilities[L["Global Cooldown"]].duration = 1.5;
		self.db.char.abilities[L["Global Cooldown"]].icon = "Interface\\Icons\\Ability_Whirlwind";
		self.db.char.abilities[L["Global Cooldown"]].group = "defaultgroup";
	end
	if not self.db.char.abilities[L["Time until Arena begins"]] then
		self.db.char.abilities[L["Time until Arena begins"]] = {};
		self.db.char.abilities[L["Time until Arena begins"]].enable = true;
		self.db.char.abilities[L["Time until Arena begins"]].duration = 60;
		self.db.char.abilities[L["Time until Arena begins"]].icon = "Interface\\Icons\\Ability_Warrior_Challange";
		self.db.char.abilities[L["Time until Arena begins"]].group = "defaultgroup";
	end
	self:RegisterEvent("CHAT_MSG_SPELL_SELF_BUFF");
	self:RegisterEvent("CHAT_MSG_SPELL_HOSTILEPLAYER_DAMAGE");
	self:RegisterEvent("CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS");
	self:RegisterEvent("PLAYER_AURAS_CHANGED");
	self:RegisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH");
	self:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_OTHER");
	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE");
	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE");
	self:RegisterEvent("CHAT_MSG_COMBAT_SELF_MISSES");
	self:RegisterEvent("CHAT_MSG_COMBAT_CREATURE_VS_SELF_MISSES");
	self:RegisterEvent("CHAT_MSG_COMBAT_HOSTILEPLAYER_MISSES", "CHAT_MSG_COMBAT_CREATURE_VS_SELF_MISSES" );
	self:RegisterEvent("CHAT_MSG_SPELL_CREATURE_VS_SELF_DAMAGE");
	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_BUFFS");
	self:RegisterEvent("CHAT_MSG_SPELL_SELF_DAMAGE");
	self:RegisterEvent("CHAT_MSG_SPELL_PET_DAMAGE");
	self:RegisterEvent("CHAT_MSG_COMBAT_SELF_HITS");
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");
	self:RegisterEvent("UNIT_SPELLCAST_SENT");
	self:RegisterEvent("SPELLS_CHANGED");
	self:RegisterEvent("SPELL_UPDATE_COOLDOWN");
	self:RegisterEvent("UNIT_AURA");
	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_BUFFS");
	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_PARTY_BUFFS", "CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_BUFFS");
	FuBar_ST:SetUp()
end

function SorrenTimers:UNIT_SPELLCAST_SENT()
	if arg1 ~= "player" then return; end
	barTarget = arg4;
end

function SorrenTimers:SPELLS_CHANGED()
	self:GetGlobalCooldownSpell()
end

function SorrenTimers:GetGlobalCooldownSpell()
	local i = 1;
	while GetSpellName(i, BOOKTYPE_SPELL) do
		if GetSpellName(i, BOOKTYPE_SPELL) == gcdspell then
			self.db.char.gcdspell = i;
			return i;
		end
		i = i + 1;
	end
end

function SorrenTimers:CHAT_MSG_BG_SYSTEM_NEUTRAL()
	if self:IsCandyBarRegistered("bgtimer") then return; end
	local bg, timeleft;
	if string.find(arg1, "Arathi Basin") then
		bg = "Arathi Basin";
	elseif string.find(arg1, "Arena") then
		bg = "Arena";
	elseif string.find(arg1, "Alterac Valley") then
		bg = "Alterac Valley";
	elseif string.find(arg1, "Warsong Gulch") then
		bg = "Warsong Gulch";
	end
	
	if string.find( string.upper(arg1), "1 MINUTE" ) or string.find( string.upper(arg1), "ONE MINUTE") then
		timeleft = 60;
	elseif string.find( string.upper(arg1), "THIRTY SECONDS" ) or string.find( string.upper(arg1), "30 SECONDS") then
		timeleft = 30;
	elseif string.find( string.upper(arg1), "FIFTEEN SECONDS" ) then
		timeleft = 15;
	end
	if not timeleft then return; end
	self:RegisterCandyBar("bgtimer", 60, "Time until "..bg.." begins", nil, unpack(self.db.char.bgSettings.neutral));
	self:RegisterCandyBarWithGroup("bgtimer", "BGTimers");
	self:SetCandyBarTexture("bgtimer", "Interface\\Addons\\SorrenTimers\\bar.tga");
	self:SetCandyBarScale("bgtimer", self.db.char.barscale);
	self:StartCandyBar("bgtimer", true);
end

function SorrenTimers:CHAT_MSG_BG_SYSTEM_HORDE()
	self:BGBars(arg1, self.db.char.bgSettings.horde);
end
function SorrenTimers:CHAT_MSG_BG_SYSTEM_ALLIANCE()
	self:BGBars(arg1, self.db.char.bgSettings.alliance);
end

function SorrenTimers:BGBars(msg, colors)
	local _,_, capped = string.find(msg, "assaulted the ([^!]+)!");
	if not capped then _, _, capped = string.find(msg, "claims the ([^!]+)!"); end
	
	if capped then
		if self:IsCandyBarRegistered(capped) then
			self:StopCandyBar(capped);
		end
		self:RegisterCandyBar(capped, 65, nodes[capped].name, nil, unpack(colors));
		self:SetCandyBarIcon(capped, "Interface\\Minimap\\POIIcons", WorldMap_GetPOITextureCoords(select(3,GetMapLandmarkInfo(nodes[capped].num)) ));
		self:RegisterCandyBarWithGroup(capped, "BGTimers");
		self:SetCandyBarTexture(capped, "Interface\\Addons\\SorrenTimers\\bar.tga");
		self:SetCandyBarScale(capped, self.db.char.barscale);
		self:StartCandyBar(capped, true);
		return;
	end
	_,_, capped = string.find(msg, "taken the ([^!]+)!");
	if capped then self:StopCandyBar(capped); end
end

function SorrenTimers:SPELL_UPDATE_COOLDOWN()
	if not self.db.char.abilities[L["Global Cooldown"]].enable then return; end
	local starttime = GetSpellCooldown(self.db.char.gcdspell, BOOKTYPE_SPELL);
	--self:Print(GetTime().." "..starttime.." "..gcdstart);
	if starttime ~= 0 and starttime ~= gcdstart then
		gcdstart = starttime;
		self:SetUpBar(self.db.char.abilities[L["Global Cooldown"]], L["Global Cooldown"]);
	end
end

function SorrenTimers:START_AUTOREPEAT_SPELL()
	if not self.db.char.abilities[BS["Auto Shot"]].enable then return; end
	self.db.char.abilities[BS["Auto Shot"]].active = true;
	self.db.char.abilities[BS["Auto Shot"]].duration = 0.5;
	if( self:IsCandyBarRegistered(BS["Auto Shot"]) )then
		self:SetCandyBarTime(BS["Auto Shot"], self.db.char.abilities[BS["Auto Shot"]].duration);
	end
	self.db.char.abilities[BS["Auto Shot"]].icon = GetInventoryItemTexture("player", 18);
	self:SetUpBar(self.db.char.abilities[BS["Auto Shot"]], BS["Auto Shot"]);
end

function SorrenTimers:STOP_AUTOREPEAT_SPELL()
	if not self.db.char.abilities[BS["Auto Shot"]].enable then return; end
	self.db.char.abilities[BS["Auto Shot"]].active = false;
	self.db.char.abilities[BS["Auto Shot"]].eatNext = nil;
end

function SorrenTimers:CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_BUFFS()
	for unit, buff in string.gmatch(arg1, L["(.+) gains (.+)%."]) do
		if enemySkills[buff] and enemySkills[buff].enable then
			self:SetUpBar(enemySkills[buff], buff, unit, true, nil, true);
		end
	end
end

function SorrenTimers:CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_BUFFS()
	for unit, buff in string.gmatch(arg1, L["(.+) gains (.+)%."]) do
		if skillSet.buffs[buff] and (GetTime() - skillSet.buffs[buff].enable < 1) and skillSet.buffs[buff].target == unit then
			skillSet.buffs[buff].enable = nil;
			self:SetUpBar(skillSet.buffs[buff], buff, unit, nil, barRank);
			skillSet.buffs[buff].target = nil;
		end
	end
end

function SorrenTimers:CHAT_MSG_COMBAT_HOSTILE_DEATH()
--"You have slain Razerfang Hatchling!"
	for unit in string.gmatch( arg1, L["You have slain (.+)!"] ) do
		for skillName, skill in pairs(skillSet.onKill) do
			if skill.enable then
				self:SetUpBar(skill, skillName, nil, nil );
			end
		end
		if units[unit] then
			for skillName,skill in pairs(units[unit]) do
				self:StopCandyBar(skillName.."-"..unit);
				if skill.diminishing then
					self:StopCandyBar(skill.diminishing.."-diminish-"..unit);
				end
			end
			units[unit] = nil;
		end
	end
	for unit in string.gmatch( arg1, L["(.+) dies%."] ) do
		if units[unit] then
			for skillName,skill in pairs(units[unit]) do
				self:StopCandyBar(skillName.."-"..unit);
				if skill.diminishing then
					self:StopCandyBar(skill.diminishing.."-diminish-"..unit);
				end
			end
			units[unit] = nil;
		end
	end
end

function SorrenTimers:CHAT_MSG_COMBAT_SELF_MISSES()
	if string.find(arg1, L["dodges"] ) then
		for skillName, skill in pairs(skillSet.onEnemyDodge) do
			if skill.enable then
				self:SetUpBar(skill, skillName, nil, nil);
			end
		end
	end
	if string.find(arg1, L["parries"] ) then
		for skillName, skill in pairs(skillSet.onEnemyParry) do
			if skill.enable then
				self:SetUpBar(skill, skillName, nil, nil);
			end
		end
	end
end

function SorrenTimers:CHAT_MSG_COMBAT_CREATURE_VS_SELF_MISSES()
	if string.find(arg1, L["You dodge"] ) then
		for skillName, skill in pairs(skillSet.onDodge) do
			if skill.enable then
				self:SetUpBar(skill, skillName, nil, nil);
			end
		end
	end
	if string.find(arg1, L["You block"] ) then
		for skillName, skill in pairs(skillSet.onBlock) do
			if skill.enable then
				self:SetUpBar(skill, skillName, nil, nil);
			end
		end
	end
	if string.find(arg1, L["You parry"] ) then
		for skillName, skill in pairs(skillSet.onParry) do
			if skill.enable then
				self:SetUpBar(skill, skillName, nil, nil);
			end
		end
	end
end

function SorrenTimers:CHAT_MSG_COMBAT_SELF_HITS()
	if string.find( arg1, L["crit"] ) then
		if( skillSet.onCrit ) then
			for skillName, skill in pairs(skillSet.onCrit) do 
				if skill.enable then
					self:SetUpBar(skill, skillName, nil, nil);
				end
			end
		end
	end
end

function SorrenTimers:CHAT_MSG_SPELL_CREATURE_VS_SELF_DAMAGE()
	if string.find(arg1, L["You dodged"] ) then
		for skillName, skill in pairs(skillSet.onDodge) do
			if skill.enable then
				self:SetUpBar(skill, skillName, nil, nil);
			end
		end
	end
	if string.find(arg1, L["You blocked"] ) then
		for skillName, skill in pairs(skillSet.onBlock) do
			if skill.enable then
				self:SetUpBar(skill, skillName, nil, nil);
			end
		end
	end
	if string.find(arg1, L["You parried"] ) then
		for skillName, skill in pairs(skillSet.onParry) do
			if skill.enable then
				self:SetUpBar(skill, skillName, nil, nil);
			end
		end
	end
end

function SorrenTimers:UNIT_SPELLCAST_FAILED()
	if not arg1 == "player" then return; end
	if barQueue then 
		if skillSet.useActionSkills[barQueue].afflict then
			if type(skillSet.useActionSkills[barQueue].afflict) == 'table' then
				for k,v in pairs(skillSet.useActionSkills[barQueue].afflict) do
					skillSet.debuffs[v].enable = false;
				end
			else
				skillSet.debuffs[skillSet.useActionSkills[barQueue].afflict].enable = false;
			end
		end
		if skillSet.useActionSkills[barQueue].aoeafflict then
			if type(skillSet.useActionSkills[barQueue].aoeafflict) == 'table' then
				for k,v in pairs(skillSet.useActionSkills[barQueue].aoeafflict) do
					skillSet.aoeDebuffs[v].enable = nil;
				end
			else
				skillSet.aoeDebuffs[skillSet.useActionSkills[barQueue].aoeafflict].enable = nil;
			end
		end
	end
	barTarget = nil;
	barQueue = nil;
end

function SorrenTimers:CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE()
	self:CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE(false);
end

function SorrenTimers:CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE(npc)
	for player, skill in string.gmatch( arg1, L["(.+) is afflicted by (.+)%."] ) do
		skill = string.gsub( skill, " %b()$", "" );
		if skillSet.aoeDebuffs[skill] and skillSet.aoeDebuffs[skill].enable then
			if (GetTime() - skillSet.aoeDebuffs[skill].enable < 1 ) then
				self:SetUpBar(skillSet.aoeDebuffs[skill], skill, player, npc, barRank);
				if skillSet.aoeDebuffs[skill].charges then
					aoeCharges = aoeCharges+1;
					if aoeCharges == skillSet.aoeDebuffs[skill].charges then
						skillSet.aoeDebuffs[skill].enable = nil;
					end
				end
			else
				skillSet.aoeDebuffs[skill].enable = nil;
			end
			barTarget = nil;
			return;
		end
		if( not skillSet.debuffs[skill] or not skillSet.debuffs[skill].enable ) then
			return
		end
		if GetTime() - skillSet.debuffs[skill].enable < 1 then
			self:SetUpBar(skillSet.debuffs[skill], skill, player, npc, barRank);
			barTarget = nil;
			skillSet.debuffs[skill].enable = nil;
		end
	end
end

function SorrenTimers:CHAT_MSG_SPELL_AURA_GONE_OTHER()
	for skill, unit in string.gmatch( arg1, L["(.+) fades from (.+)%."] ) do
		if units[unit] and units[unit][skill] then
			self:StopCandyBar(skill.."-"..unit);
			if units[unit][skill].diminishing then
				units[unit][units[unit][skill].diminishing].last = GetTime();
				self:StartCandyBar(units[unit][skill].diminishing.."-diminish-"..unit);
			end
		end
	end
end

function SorrenTimers:CHAT_MSG_SPELL_SELF_DAMAGE()
	if string.find( arg1, L["missed"] ) then
		for skill, unit in string.gmatch( arg1, L["Your (.+) missed (.+)%."] ) do
			if skill == BS["Auto Shot"] then return end
			if self:IsCandyBarRegistered(skill) then
				self:StopCandyBar(skill);
			end
			if self:IsCandyBarRegistered(skill.."-"..unit) then
				self:StopCandyBar(skill.."-"..unit);
			end
			
		end
	else
		if interruptSkill and string.find(arg1, L["You interrupt"]) then
		--self:Print("here");
			for unit in string.gmatch(arg1, L["interrupt (.+)'s"] ) do
				self:SetUpBar(interruptSkill, interruptSkill.name, unit, nil);
			end
			interruptSkill = nil;
			return;
		end
		local hits = L["hits"];
		if( string.find( arg1, L["crits"] ) ) then
			hits = L["crits"];
			if( skillSet.onCrit ) then
				for skillName, skill in pairs(skillSet.onCrit) do 
					if skill.enable then
						local _,_,name,target = string.find(arg1, L["Your (.+) crits (.+) for .+%."]);
						if not skill.check or (skill.check and skill.check[name]) then
							self:SetUpBar(skill, skillName, name, UnitIsPlayer("target"));
						end
					end
				end
			end
		end
		
		for skill, unit in string.gmatch( arg1, L["Your (.+) "..hits.." (.+) for .+%."] ) do
			if skillSet.castDamageSpells[skill] and skillSet.castDamageSpells[skill].enable then
				self:SetUpBar(skillSet.castDamageSpells[skill], skill, unit, UnitIsPlayer("target") );
			end
			return;
		end
		for skill in string.gmatch( arg1, L["You cast (.+)%."] ) do
			if skillSet.castDamageSpells[skill] and skillSet.castDamageSpells[skill].enable then
				self:SetUpBar(skillSet.castDamageSpells[skill], skill);
			end
		end
	end

end

function SorrenTimers:CHAT_MSG_SPELL_PET_DAMAGE()
	local regex;
	if( string.find( arg1, L["fails"] ) ) then
		regex = L["'s (.+ Effect) fails%. .+ is immune%."];
	else
		regex = L["'s (.+ Effect) was resisted by .+%."];
	end
	for trapeffect in string.gmatch( arg1, regex ) do
		if skillSet.debuffs[trapeffect] and skillSet.debuffs[trapeffect].enable then
			skillSet.debuffs[trapeffect].enable = false;
			if skillSet.debuffs[trapeffect].stop then self:StopCandyBar(skillSet.debuffs[trapeffect].stop); end
		end
	end
end

function SorrenTimers:PLAYER_COMBO_POINTS()
	if GetComboPoints() ~= 0 then
		self.comboPoints = GetComboPoints();
	end
end


function SorrenTimers:OnDisable()

end

function SorrenTimers:CHAT_MSG_SPELL_SELF_BUFF()
	for skill, unit in string.gmatch( arg1, L["Your (.+) heals (.+) for .+%."] ) do
		if skillSet.castSpells[skill] and skillSet.castSpells[skill].enable then
			if( unit == L["you"] ) then unit = L["You"]; end
			self:SetUpBar(skillSet.castSpells[skill], skill, unit, nil);
		end
		return;
	end
	for skill in string.gmatch( arg1, L["You cast (.+)%."] ) do
		if not skillSet.castSpells[skill] or not skillSet.castSpells[skill].enable then return; end
		if skillSet.castSpells[skill].canDie then skillSet.canDie[skillSet.castSpells[skill].canDie].enable = true; end
		self:SetUpBar(skillSet.castSpells[skill], skill, nil, nil);
	end
end

function SorrenTimers:CreateAnchor(group)
	if not self.db.char.anchors then self.db.char.anchors = {}; end	
	if not self.db.char.anchors["STAnchor"..group] then 
		self.db.char.anchors["STAnchor"..group] = {
			["point"] = "CENTER",
			["relPoint"] = "CENTER",
			["x"] = 0,
			["y"] = 0,
			["unlocked"] = true;
		};
	end
	if not self.anchors then self.anchors = { }; end
	
	self.anchors[group] = CreateFrame("Frame",group..tostring(math.floor(GetTime())),UIParent)
	self.anchors[group]:ClearAllPoints();
	self.anchors[group]:SetBackdrop( {
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile="Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true, tileSize = 16, edgeSize = 16, insets = {
			left = 5, right = 5, top = 5, bottom = 5 },
	} );
	self.anchors[group]:SetWidth(128)
	self.anchors[group]:SetHeight(30)
	self.anchors[group]:SetBackdropColor(0,0,0);
	self.anchors[group]:EnableMouse(true);
	self.anchors[group]:SetPoint(self.db.char.anchors["STAnchor"..group].point, 
		"UIParent", 
		self.db.char.anchors["STAnchor"..group].relPoint, 
		self.db.char.anchors["STAnchor"..group].x, 
		self.db.char.anchors["STAnchor"..group].y);
	local msg = self.anchors[group]:CreateFontString(nil,"overlay","GameFontNormal");
	msg:ClearAllPoints();
	msg:SetPoint("center", self.anchors[group], "center");
	msg:SetText(group);
	self.anchors[group]:SetMovable(true);
	self.anchors[group]:SetScript("OnDragStart", function() self.anchors[group]:StartMoving(); GameTooltip:Hide() end);
	self.anchors[group]:SetScript("OnDragStop", function() self.anchors[group]:StopMovingOrSizing(); SorrenTimers:SavePosition(group) end);
	self.anchors[group]:SetScript("OnMouseUp", function() self:OnAnchorClick(group) end);
	self.anchors[group]:SetScript("OnEnter", function() self:ShowAnchorTooltip() end);
	self.anchors[group]:SetScript("OnLeave", function() GameTooltip:Hide() end);
	self.anchors[group]:RegisterForDrag("LeftButton");
	if self.db.char.anchors["STAnchor"..group].unlocked then
		self.anchors[group]:Show();
	else
		self.anchors[group]:Hide();
	end
end

function SorrenTimers:ShowAnchorTooltip()
	GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT");
	GameTooltip:AddLine("Sorren Timers Group Anchor");
	GameTooltip:AddLine("Drag this to reposition");
	GameTooltip:AddLine("Shift+Left Click to hide");
	GameTooltip:AddLine("Alt+Left Click to show a test bar");
	GameTooltip:AddLine("Alt+Left Click again to hide the test bar");
	GameTooltip:AddLine("If you hide this, you can show it again by going to Group Options -> groupname -> Uncheck Lock");
	GameTooltip:Show();
end



function SorrenTimers:OnAnchorClick(group, altkey)
	if IsAltKeyDown() or altkey then
		if self:IsCandyBarRegistered(group.."testbar") then
			self:StopCandyBar(group.."testbar");
			return;
		end
		self:RegisterCandyBar(group.."testbar", 300, "Test bar", "Interface\\Icons\\INV_Misc_QuestionMark", unpack(self.db.char.defaultcolors) );
		self:RegisterCandyBarWithGroup(group.."testbar", group);
		self:SetCandyBarTexture(group.."testbar", "Interface\\Addons\\SorrenTimers\\bar.tga");
		self:SetCandyBarScale(group.."testbar", self.db.char.barscale);
		self:StartCandyBar(group.."testbar", true);
	end
	if IsShiftKeyDown() then
		self:LockAnchor(group);
		return;
	end
end

function SorrenTimers:SavePosition(group)
	self.db.char.anchors["STAnchor"..group].point, _, self.db.char.anchors["STAnchor"..group].relPoint, self.db.char.anchors["STAnchor"..group].x, self.db.char.anchors["STAnchor"..group].y = self.anchors[group]:GetPoint();
end

function SorrenTimers:PLAYER_AURAS_CHANGED()
	--if not skillSet.onSelfBuff then self:UnregisterEvent("PLAYER_AURAS_CHANGED"); return; end
	if not self.updateFrame then 
		self.updateFrame = CreateFrame("Frame", nil, UIParent);
		self.updateFrame.onUpdate = function(elapsed)
			this.elapsed = this.elapsed + elapsed;
			if this.elapsed < 0.5 then return; end
			this.elapsed = 0;
			for k,v in pairs(units.player) do
				if k ~= GetPlayerBuffName(v.id) then break; end
				if v.update and GetPlayerBuffTimeLeft(v.id) > v.timeleft then
					skillSet.onSelfBuff[k].duration = GetPlayerBuffTimeLeft(v.id);
					self:SetUpBar(skillSet.onSelfBuff[k], k);
				end
				v.timeleft = GetPlayerBuffTimeLeft(v.id)
			end
		end
		self.updateFrame.elapsed = 0;
		self.updateFrame:SetScript("OnUpdate", function()
			self.updateFrame.onUpdate(arg1)
		end );
	end
	local i = 1;
	local name;
	if not units.player then units.player = {}; end
	for k,v in pairs(units.player) do
		v.found = false;
	end
	for k,v in pairs(auras) do
		auras[k].found = false;
	end
	local buff = GetPlayerBuff(i);
	while buff ~= 0 do
		name = GetPlayerBuffName(i);
		if skillSet.onSelfBuff[name] and skillSet.onSelfBuff[name].enable then
			self.updateFrame:Show();
			skillSet.onSelfBuff[name].duration = GetPlayerBuffTimeLeft(i);
			if not units.player[name] then units.player[name] = { ["update"] = true };	end
			units.player[name].found, units.player[name].id, units.player[name].timeleft = true, i, skillSet.onSelfBuff[name].duration;
			if not skillSet.onSelfBuff[name].icon then skillSet.onSelfBuff[name].icon = GetPlayerBuffTexture(i); end
			self:SetUpBar(skillSet.onSelfBuff[name], name);
		end
		if auras[GetPlayerBuffTexture(i)] then
			auras[GetPlayerBuffTexture(i)].found = true;
		end
		i = i + 1;
		buff = GetPlayerBuff(i);
	end
	for k,v in pairs(units.player) do
		if not v.found then
			self:StopCandyBar(k);
			units.player[k] = nil;
		end
	end
	for k,v in pairs(auras) do
		if( not v.found ) then
			if type(v.stop) == 'table' then
				for i,j in pairs(v.stop) do
					self:StopCandyBar(j);
				end
			else
				self:StopCandyBar(v.stop);
			end
			v.enable = false;
			auras[k] = nil;
		end
	end
	if not next(units.player) then
		self.updateFrame:Hide();
	end
end

function SorrenTimers:FindBuff(buff)
	local i = 0;
	local tex = GetPlayerBuffTexture(i);
	while tex do
		if tex == buff then return true; end
		i = i + 1;
		tex = GetPlayerBuffTexture(i);
	end
	return false;
end

function SorrenTimers:UNIT_AURA()
	if arg1 ~= "target" then return end
	local i = 1;
	unit = UnitName("target");
	while UnitDebuff("target",i) do
		name = UnitDebuff("target",i);
		if skillSet.onDebuff[name] and skillSet.onDebuff[name].enable then
			if not units[unit] or not units[unit][name] or (GetTime() - units[unit][name].last > skillSet.onDebuff[name].duration) then
				self:SetUpBar(skillSet.onDebuff[name], name, unit, UnitIsPlayer("target"));
			end
		end
		i = i + 1;
	end
end

function SorrenTimers:CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS()
	self:CHAT_MSG_SPELL_HOSTILEPLAYER_DAMAGE()
end
function SorrenTimers:CHAT_MSG_SPELL_HOSTILEPLAYER_DAMAGE()
	hits = L["hits"];
	if string.find( arg1, "crits" ) then
		hits = L["crits"];
	end
	
	if string.find(arg1, L["You dodged"] ) then
		for skillName, skill in pairs(skillSet.onDodge) do
			if skill.enable then
				self:SetUpBar(skill, skillName, nil, nil);
			end
		end
	end
	if string.find(arg1, L["You blocked"] ) then
		for skillName, skill in pairs(skillSet.onBlock) do
			if skill.enable then
				self:SetUpBar(skill, skillName, nil, nil);
			end
		end
	end
	if string.find(arg1, L["You parried"] ) then
		for skillName, skill in pairs(skillSet.onParry) do
			if skill.enable then
				self:SetUpBar(skill, skillName, nil, nil);
			end
		end
	end
	for skill in string.gmatch( arg1, L[".+ "..hits.." (.+) for"] ) do
		skill = string.gsub( skill, L[" [IV]+$"], "" );
		if skillSet.canDie[skill] and skillSet.canDie[skill].enable then
			if (not skillSet.canDie[skill].aura) or (not self:FindBuff(skillSet.canDie[skill].aura)) then
				if type(skillSet.canDie[skill].stop) == 'table' then
					for k,v in pairs(skillSet.canDie[skill].stop) do
						self:StopCandyBar(v);
					end
				else
					self:StopCandyBar(skillSet.canDie[skill].stop);
				end
				skillSet.canDie[skill].enable = false;
			else
				auras[skillSet.canDie[skill].aura] = { };
				auras[skillSet.canDie[skill].aura].stop = skillSet.canDie[skill].stop;
				auras[skillSet.canDie[skill].aura].enable = skillSet.canDie[skill].enable; 
			end

		end
		return;
	end
	
	if string.find( arg1, "immune" ) then
		for skill in string.gmatch( arg1, L["fails%. (.+) is immune"] ) do
			if skillSet.canDie[skill] and skillSet.canDie[skill].special and skillSet.canDie[skill].enable and skillSet.canDie[skill].aura and not self:FindBuff(skillSet.canDie[skill].aura) then
				self:SetUpBar(skillSet.special[skillSet.canDie[skill].special], skillSet.canDie[skill].special, nil, nil);
			end
		end
	end
end

function SorrenTimers:SetUpBar(skill, skillName, unit, notnpc, rank, enemyskill)
	self:Debug(skillName, unit, rank);
	if not enemyskill then
		if not self.db.char.abilities[skillName].enable then return; end
	else
		if not self.db.char.enemy[skillName].enable then return; end
	end
	local skilldisplay = skillName;
	if not enemyskill then
		if self.db.char.abilities[skillName].name then skilldisplay = self.db.char.abilities[skillName].name; end
	else
		if self.db.char.enemy[skillName].name then skilldisplay = self.db.char.enemy[skillName].name; end
	end
	local mult = 1;
	
	if skill.ranks and rank then
		for num in string.gmatch(rank, L["Rank (%d+)"]) do
			num = tonumber(num);
			if type(skill.ranks[num]) ~= 'table' then
				skill.duration = skill.ranks[num];
			else
				skill.points = skill.ranks[num].points;
			end
		end
	end
	if skill.points then
		self:Debug("Points: ",self.comboPoints);
		skill.duration = skill.points[self.comboPoints];
	end
	if skill.stopother then
		for unitName, skills in pairs(units) do
			for skillname, uskill in pairs(skills) do
				if skillname == skillName then
					self:StopCandyBar(skillName.."-"..unitName);
					uskill = nil;
				end
			end
			if not next(skills) then
				skills = nil;
			end
		end
	end
	local duration = skill.duration;
	if unit and skill.target then
		if not units[unit] then units[unit] = {}; end
		if not units[unit][skillName] then units[unit][skillName] = {}; end
		if (skill.diminishing and notnpc) or skill.alwaysdiminish then
			units[unit][skillName].diminishing = skill.diminishing;
			if duration and duration > 12 then duration = 12; end
			if not skill.refresh or (skill.refresh and self:IsCandyBarRegistered(skill.diminishing.."-"..unit)) then
				if not units[unit][skill.diminishing] then units[unit][skill.diminishing] = {}; end
				if not units[unit][skill.diminishing].last or (GetTime() - units[unit][skill.diminishing].last > 20 ) then
					units[unit][skill.diminishing].mult = 1;
					units[unit][skill.diminishing].casts = 2;
					units[unit][skill.diminishing].last = GetTime()+(duration or 0);
				else
					units[unit][skill.diminishing].mult = units[unit][skill.diminishing].mult * 0.5;
					mult = units[unit][skill.diminishing].mult;
					units[unit][skill.diminishing].casts = units[unit][skill.diminishing].casts - 1;
				end
				local group = self.db.char.diminish.group;
				if self.db.char.abilities[skill.diminishing].group then
					group = self.db.char.abilities[skill.diminishing].group;
				end
				if not group then group = skill.group; end
				if not group then group = self.db.char.abilities[skillName].group; end;
				if not self.db.char.abilities[skill.diminishing].color then
					if self.db.char.groups[skill.group].color then
						colors = self.db.char.groups[skill.group].color;
					else
						colors = self.db.char.diminish.colors;
					end
				else
					colors = self.db.char.abilities[skill.diminishing].color;
				end
				local barfade = self.db.char.barfade;
				if self.db.char.abilities[skillName].fade then
					barfade = self.db.char.abilities[skillName].fade;
				end
				skillstring = string.gsub(self.db.char.diminish.skillstring, "$name", skill.diminishing);
				skillstring = string.gsub(skillstring, "$target", unit);
				skillstring = string.gsub(skillstring, "$casts", units[unit][skill.diminishing].casts);
				if( not self:IsCandyBarRegistered(skill.diminishing.."-diminish-"..unit)) then
					self:RegisterCandyBar(skill.diminishing.."-diminish-"..unit, 20, skillstring, skill.icon, unpack(colors) );
					self:SetCandyBarTexture(skill.diminishing.."-diminish-"..unit, "Interface\\Addons\\SorrenTimers\\bar.tga");
					self:RegisterCandyBarWithGroup(skill.diminishing.."-diminish-"..unit, group);	
					self:SetCandyBarScale(skill.diminishing.."-diminish-"..unit, self.db.char.barscale);
					self:SetCandyBarFade(skill.diminishing.."-diminish-"..unit, barfade);
				else
					self:SetCandyBarText(skill.diminishing.."-diminish-"..unit, skillstring );
					self:SetCandyBarTimeLeft(skill.diminishing.."-diminish-"..unit, 20);
				end
				self:StartCandyBar(skill.diminishing.."-diminish-"..unit, true);
				self:PauseCandyBar(skill.diminishing.."-diminish-"..unit);
			end
		end
		units[unit][skillName].last = GetTime();
		if( skill.refresh ) and self:IsCandyBarRegistered(skill.refresh.."-"..unit) then
			if skill.afflict then
				self:SetCandyBarTimeLeft(skill.refresh.."-"..unit, skillSet.debuffs[skill.refresh].duration * mult);
			end
			if skill.aoeafflict then
				self:SetCandyBarTimeLeft(skill.refresh.."-"..unit, skillSet.aoeDebuffs[skill.refresh].duration * mult);
			end
		end
	else
		if skill.refresh and self:IsCandyBarRegistered(skill.refresh) then
			if skill.afflict then
				self:SetCandyBarTimeLeft(skill.refresh, skillSet.debuffs[skill.refresh].duration);
			end
			if skill.aoeafflict then
				self:SetCandyBarTimeLeft(skill.refresh, skillSet.aoeDebuffs[skill.refresh].duration);
			end
		end		
	end
	if duration then	
		if not enemyskill then
			if not self.db.char.groups[skill.group] then
				self.db.char.abilities[skillName].group = "defaultgroup";
			end
		else
			if not self.db.char.groups[skill.group] then
				self.db.char.enemy[skillName].group = "defaultgroup";
			end
		end
		local skillstring = self.db.char.skillstring;
		if skill.target then
			skillstring = self.db.char.skilltargetstring;
		end
		local colors;
		if not enemyskill then
			if not self.db.char.abilities[skillName].color then
				if self.db.char.groups[skill.group].color then
					colors = self.db.char.groups[skill.group].color;
				else
					colors = self.db.char.defaultcolors;
				end
			else
				colors = self.db.char.abilities[skillName].color;
			end
		else
			if not self.db.char.enemy[skillName].color then
				if self.db.char.groups[skill.group].color then
					colors = self.db.char.groups[skill.group].color;
				else
					colors = self.db.char.defaultcolors;
				end
			else
				colors = self.db.char.enemy[skillName].color;
			end
			skillstring = self.db.char.enemyskillstring;
		end
		local barfade = self.db.char.barfade;
		if not enemyskill then
			if self.db.char.abilities[skillName].fade then
				barfade = self.db.char.abilities[skillName].fade;
			end
		else
			if self.db.char.enemy[skillName].fade then
				barfade = self.db.char.enemy[skillName].fade;
			end
		end
		skillstring = string.gsub(skillstring, "$name", skilldisplay);
		skillstring = string.gsub(skillstring, "$target", unit);
		if skill.target and unit then
			self:RegisterCandyBar(skillName.."-"..unit, duration * mult, skillstring, skill.icon, unpack(colors) );
			self:SetCandyBarCompletion(skillName.."-"..unit, Clean, skillName, unit, skill.diminishing);
			self:SetCandyBarTexture(skillName.."-"..unit, "Interface\\Addons\\SorrenTimers\\bar.tga");
			self:RegisterCandyBarWithGroup(skillName.."-"..unit, skill.group);
			self:SetCandyBarScale(skillName.."-"..unit, self.db.char.barscale);
			self:SetCandyBarFade(skillName.."-"..unit, barfade);
			self:StartCandyBar(skillName.."-"..unit, true);
		elseif( skill.multiple ) then
			local instance = GetTime();
			self:RegisterCandyBar( skillName..instance, duration * mult, skillstring, skill.icon, unpack(colors) );
			self:SetCandyBarTexture(skillName..instance, "Interface\\Addons\\SorrenTimers\\bar.tga");
			self:RegisterCandyBarWithGroup(skillName..instance, skill.group);
			self:SetCandyBarScale(skillName..instance, self.db.char.barscale);
			self:SetCandyBarFade(skillName..instance, barfade);
			self:StartCandyBar(skillName..instance, true);
		else
			if not self:IsCandyBarRegistered(skillName) then
				self:RegisterCandyBar(skillName, duration * mult, skillstring, skill.icon, unpack(colors) );
				self:SetCandyBarTexture(skillName, "Interface\\Addons\\SorrenTimers\\bar.tga");
				self:RegisterCandyBarWithGroup(skillName, skill.group);
			end
			self:SetCandyBarScale(skillName, self.db.char.barscale);
			self:SetCandyBarFade(skillName, barfade);
			self:StartCandyBar(skillName, true);
		end
	end
	if skill.start then
		if type(skill.start) == 'table' then
			for k,v in pairs(skill.start) do
				if skillSet.special[v] then
					skillSet.special[v].icon = skill.icon;
					skillSet.special[v].group = skill.group;
					self:SetUpBar(skillSet.special[v], v, nil, nil, rank);
				end
			end
		else
			if skillSet.special[skill.start] then
				skillSet.special[skill.start].icon = skill.icon;
				skillSet.special[skill.start].group = skill.group;
				self:SetUpBar(skillSet.special[skill.start], skill.start, nil, nil, rank);
			end
		end
	end
	if skill.stop then 
		if type(skill.stop) == 'table' then
			for k,v in pairs(skill.stop) do
				if( not skill.stopunit ) then
					self:StopCandyBar(v);
				else
					self:StopCandyBar(v.."-"..unit);
				end
			end
		else
			if( not skill.stopunit ) then
				self:StopCandyBar(skill.stop);
			else
				self:StopCandyBar(skill.stop.."-"..unit);
			end
		end
	end
	if skill.disable then 
		if type(skill.disable) == 'table' then
			for k,v in pairs(skill.disable) do
				skillSet.debuffs[v].enable = false;
			end
		else
			skillSet.debuffs[skill.disable].enable = false; 
		end
	end
end

function SorrenTimers:AbsorbTable(table1, absorbed)
	for k,v in pairs(absorbed) do
		if type(v) == 'table' then
			table1[k] = {};
			self:AbsorbTable(table1[k], v);
		else
			table1[k] = v;
		end
	end
end


